/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2020 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: Internal                                         *
*                                                                    *
*       Current version number will be inserted here                 *
*       when shipment is built.                                      *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : Initializes and handles the hardware for embOS
*/

#include "RTOS.h"
#include "altera_avalon_timer_regs.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/*********************************************************************
*
*       System tick settings
*/
#if (ALT_SYS_CLK == SYS_CLK_TIMER)
  #define OS_TIMER_FREQ  SYS_CLK_TIMER_FREQ
  #define OS_TIMER_BASE  SYS_CLK_TIMER_BASE
  #define OS_TICK_FREQ   SYS_CLK_TIMER_TICKS_PER_SEC
#else
  #define OS_TIMER_FREQ  HIGH_RES_TIMER_FREQ
  #define OS_TIMER_BASE  HIGH_RES_TIMER_BASE
  #define OS_TICK_FREQ   HIGH_RES_TIMER_TICKS_PER_SEC
#endif

#define OS_INT_FREQ    (OS_TICK_FREQ)

/*********************************************************************
*
*       embOSView settings
*/
#ifndef   OS_VIEW_IFSELECT
  #define OS_VIEW_IFSELECT  OS_VIEW_DISABLED
#endif

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  #include "altera_avalon_uart.h"
  #include "altera_avalon_uart_regs.h"
  #define OS_UART      (0u)
  #define OS_BAUDRATE  (115200uL)
#endif

/*********************************************************************
*
*       System stack size
*/
#ifndef   OS_SYS_STACK_SIZE
  #define OS_SYS_STACK_SIZE  (768u)
#endif

/*********************************************************************
*
*       Interrupt nesting
*/
#define ALLOW_NESTED_INTERRUPTS  (0)

#if ALLOW_NESTED_INTERRUPTS
  #include "priv/alt_legacy_irq.h"
#endif

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

/*********************************************************************
*
*       _OS_GetHWTimerCycles()
*
*  Function description
*    Returns the current hardware timer count value.
*
*  Return value
*    Current timer count value.
*/
static unsigned int _OS_GetHWTimerCycles(void) {
  IOWR_ALTERA_AVALON_TIMER_SNAPH(OS_TIMER_BASE, 0);  /* Take new snapshot of timer value */
  return (IORD_ALTERA_AVALON_TIMER_SNAPH(OS_TIMER_BASE) << 16) + IORD_ALTERA_AVALON_TIMER_SNAPL(OS_TIMER_BASE);
}

/*********************************************************************
*
*       _OS_GetHWTimer_IntPending()
*
*  Function description
*    Returns if the hardware timer interrupt pending flag is set.
*
*  Return value
*    == 0: Interrupt pending flag not set.
*    != 0: Interrupt pending flag set.
*/
static unsigned int _OS_GetHWTimer_IntPending(void) {
  return (IORD_ALTERA_AVALON_TIMER_STATUS(OS_TIMER_BASE) & ALTERA_AVALON_TIMER_STATUS_TO_MSK);
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_STACK_GetSysStackSize()
*
*  Function description
*    Returns the system stack size.
*
*  Return value
*    System stack size.
*/
unsigned int OS_STACK_GetSysStackSize(void) {
  return OS_SYS_STACK_SIZE;
}

/*********************************************************************
*
*       OS_InitHW()
*
*  Function description
*    Initialize the hardware required for embOS to run.
*/
const OS_SYSTIMER_CONFIG SysTimerConfig = {OS_TIMER_FREQ, OS_INT_FREQ, OS_TIMER_DOWNCOUNTING, _OS_GetHWTimerCycles, _OS_GetHWTimer_IntPending};

void OS_InitHW(void) {
  OS_INT_IncDI();
  OS_TIME_ConfigSysTimer(&SysTimerConfig);
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  OS_COM_Init();
#endif
  OS_INT_DecRI();
}

/*********************************************************************
*
*       OS_ISR_Tick()
*
*  Function description
*    This is the hardware timer exception handler.
*/
void OS_ISR_Tick(void) {
  OS_TICK_Handle();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Function description
*    This code is executed whenever no task, software timer, or
*    interrupt is ready for execution.
*
*  Additional information
*    The idle loop does not have a stack of its own, therefore no
*    functionality should be implemented that relies on the stack
*    to be preserved.
*/
void OS_Idle(void) {  // Idle loop: No task is ready to execute
  while (1) {         // Nothing to do ... wait for interrupt
  }
}

/*********************************************************************
*
*       Optional communication with embOSView
*
**********************************************************************
*/

#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)

#define OS_UART_BASE         UART1_BASE
#define OS_COM_IRQ           UART1_IRQ
#define UART_RX_ERROR_FLAGS (ALTERA_AVALON_UART_STATUS_PE_MSK  | \
                             ALTERA_AVALON_UART_STATUS_FE_MSK  | \
                             ALTERA_AVALON_UART_STATUS_BRK_MSK | \
                             ALTERA_AVALON_UART_STATUS_ROE_MSK | \
                             ALTERA_AVALON_UART_STATUS_TOE_MSK )

/*********************************************************************
*
*       OS_COM_Isr()
*
*  Function description
*    interrupt handler for UART for communication with embOSView
*/
static void OS_COM_Isr(void* context, alt_u32 id) {
  unsigned int UartStatus;
  unsigned int UartControl;
  unsigned int Data;

#if ALLOW_NESTED_INTERRUPTS
  OS_INT_EnterNestable();
#endif
  UartStatus  = IORD_ALTERA_AVALON_UART_STATUS(OS_UART_BASE);    // Get UART Status
  UartControl = IORD_ALTERA_AVALON_UART_CONTROL(OS_UART_BASE);   // Get UART control settings
  UartStatus &= UartControl;                                     // Extract valid interrupt flags
  IOWR_ALTERA_AVALON_UART_STATUS(OS_UART_BASE, 0);               // Clear any error flags
  if ((UartStatus & ALTERA_AVALON_UART_STATUS_RRDY_MSK) != 0) {  // Rx interrupt pending?
    Data = IORD_ALTERA_AVALON_UART_RXDATA(OS_UART_BASE);         // Read data
    if ((UartStatus & UART_RX_ERROR_FLAGS) == 0) {               // if no error detected
      OS_COM_OnRx(Data);                                         // process Data
    }
  }
  if ((UartStatus & ALTERA_AVALON_UART_STATUS_TRDY_MSK) != 0) {  // Tx interrupt pending?
    if (OS_COM_OnTx() != 0) {                                    // No more characters to send?
      UartControl &= ~ALTERA_AVALON_UART_CONTROL_TRDY_MSK;       // Disable Tx interrupt
      IOWR_ALTERA_AVALON_UART_CONTROL(OS_UART_BASE, UartControl);
    }
  }
#if ALLOW_NESTED_INTERRUPTS
  OS_INT_LeaveNestable();
#endif
}

/*********************************************************************
*
*       OS_COM_Init()
*
*  Function description
*    Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  //
  // Assume that UART is already functioning and baudrate is fixed
  // => nothing to initialize
  //
  // Finally install interrupt, enable UART interrupt for Rx
  //
  alt_irq_register(OS_COM_IRQ, (void*)OS_UART_BASE, OS_COM_Isr);
  IOWR_ALTERA_AVALON_UART_CONTROL(OS_UART_BASE, ALTERA_AVALON_UART_CONTROL_RRDY_MSK);
}

#endif  // (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description
*    Sends one character.
*/
void OS_COM_Send1(OS_U8 c) {
#if (OS_VIEW_IFSELECT == OS_VIEW_IF_UART)
  int UartControl;
  IOWR_ALTERA_AVALON_UART_TXDATA(OS_UART_BASE, c);       /* Write data */
  UartControl = IORD_ALTERA_AVALON_UART_CONTROL(OS_UART_BASE);
  UartControl |= ALTERA_AVALON_UART_CONTROL_TRDY_MSK;    /* Enable Tx interrupt */
  IOWR_ALTERA_AVALON_UART_CONTROL(OS_UART_BASE, UartControl);
#endif
}

/*************************** End of file ****************************/
